/********************************************************************************
*                                                                     
*       Copyright (C) 2008 Freescale Semiconductor, Inc.              
*       All Rights Reserved								              
*														              
* Filename:     main.c                
*														              
* Revision:      										              
*														              
* Functions:    Includes main function
*         
* Description:  
*
* Notes:        
*
*********************************************************************************/

#include <hidef.h> /* for EnableInterrupts macro */
#include "derivative.h" /* include peripheral declarations */
#include "porting_support.h"
#include "ICS.h"
#include "ADC16.h"
#include "interrupts.h"

/*********************************** Defines ***********************************/

#define BUSCLK_FREQ_KHZ		20000L	
#define TERMINAL_BAUD_RATE	9600 
#define SCIBD_BAUD_RATE ((BUSCLK_FREQ_KHZ * 10) / (16 * (TERMINAL_BAUD_RATE/100)))

#define ON  0xFF
#define OFF 0x00
#define SET 0xFF
#define CLEAR 0x00

/*********************************** Function **********************************/

void System_Init(void);

/*******************************************************************************
* Function:        main
*
* Description:     
*
* Returns:         never return
*
* Notes:           system, ICS, SCI1 and ADC1-4 initialised. No application code.
*
*                  ADCH -> ADCn ->->-> MUX -> Pin# -> MCU Port Header
*                  DADP0   3           n/a    22      29
*                  DADP1   1           n/a    25      31
*                  DADP2   2           n/a    28      33
*                  DADP3   4           n/a    31      35
*                  DADM0   3           n/a    23      30
*                  DADM1   1           n/a    26      32
*                  DADM2   2           n/a    29      34
*                  DADM3   4           n/a    32      36
*                  AD4     3           n/a    24      25
*                  AD5     1           n/a    27      27
*                  AD6     2           n/a    30      22
*                  AD7     3 & 4       n/a    33      24
*                  AD8     3           E0     42      41
*                  AD9     3           E1     43      43
*                  AD10    1           A2     45      19
*                  AD11    1           A5     48      13
*                  AD12    1           A6     49      15
*                  AD13    3 & 1       A7     50      39
*                  AD14    2           B6     64      9
*                  AD15    2           B7     65      11
*                  AD16    2           F4     16      *39(LCD_EN J26)
*                  AD17    4 & 2       F4     17      *37(LCD_EN J26)
*                  AD18    4 & 2       F4     18      *35(LCD_EN J26)
*                  AD19    4 & 2       F4     19      *33(LCD_EN J26)
*                 
*******************************************************************************/

void main(void) {

  volatile unsigned short data;
  volatile unsigned short MODE;
  volatile unsigned short DADn;

  MODE = MODE_10;       /* Select Conversion Mode: MODE_16, MODE_12, MODE_10, MODE_8 */
  DADn = 0;             /* Select Differential Pair: 0->3 */

  System_Init();      /* including pin mux control, ADC channel or I/O */
  /* **NEED TO UPDATE FOR SCI AND PDB AT LATER DATE** */
  
  DisableInterrupts; /* enable interrupts */
  
  /* 20MHz FEI mode */
  ICS_Config((CLKS_FLL|IREFS_INTERNAL|IRCLKEN_ACTIVE|IREFSTEN_INTERNAL_STOP_DISABLE),
  (BDIV_1|LP_FLL_ENABLED|EREFS_OSCILLATOR|ERCLKEN_ACTIVE|EREFSTEN_INTERNAL_STOP_ENABLE), 0xF0,(DCO_RANGE_HIGH|DCO_MAX_25)); 
    
  /* blank interrupt routines in interrupts.c */ 
  /* please refer to table at top of file for ADC Channel -> ADC number correlataion */
  
  if (DADn == 1){
    /* Configure ADC1 */
    ADCPointer = (tADC *)ADC1;
    ADC_Init(ADCPointer, (ADLPC_NORMAL|ADIV_8|ADLSMP_SHORT|MODE|ADICLK_BUS_2), (ADACKEN_ENABLED|ADHSC_HISPEED|ADLSTS_2) , 0, 0,
    (ADTRG_SW|ACFE_DISABLED|ACFGT_LESS|ACREN_DISABLED|REFSEL_EXT), (ADCO_SINGLE|AVGE_DISABLED), (AIEN_OFF|DIFF_DIFFERENTIAL|0x1F),
    (AIEN_OFF|DIFF_DIFFERENTIAL|0x1F));
    
    for(;;){
      ADC1SC1A_ADCHA = 0x01;
    
    	do{
    	/* Wait for conversion to complete */
    	}while (ADC1SC2_ADACT == 1);	
      
      data = ADC1RA;
    } /* loop forever */
  }
  
  if (DADn == 2){
    /* Configure ADC2 */
    ADCPointer = (tADC *)ADC2;
    ADC_Init(ADCPointer, (ADLPC_NORMAL|ADIV_8|ADLSMP_SHORT|MODE|ADICLK_BUS_2), (ADACKEN_ENABLED|ADHSC_HISPEED|ADLSTS_2) , 0, 0,
    (ADTRG_SW|ACFE_DISABLED|ACFGT_LESS|ACREN_DISABLED|REFSEL_EXT), (ADCO_SINGLE|AVGE_DISABLED), (AIEN_OFF|DIFF_DIFFERENTIAL|0x1F),
    (AIEN_OFF|DIFF_DIFFERENTIAL|0x1F));
        
    for(;;){
      ADC2SC1A_ADCHA = 0x02;
    
    	do{
    	/* Wait for conversion to complete */
    	}while (ADC2SC2_ADACT == 1);	
      
      data = ADC2RA;
    } /* loop forever */
  }
  
  if (DADn == 0){
    /* Configure ADC3 */
    ADCPointer = (tADC *)ADC3;
    ADC_Init(ADCPointer, (ADLPC_NORMAL|ADIV_8|ADLSMP_SHORT|MODE|ADICLK_BUS_2), (ADACKEN_ENABLED|ADHSC_HISPEED|ADLSTS_2) , 0, 0,
    (ADTRG_SW|ACFE_DISABLED|ACFGT_LESS|ACREN_DISABLED|REFSEL_EXT), (ADCO_SINGLE|AVGE_DISABLED), (AIEN_OFF|DIFF_DIFFERENTIAL|0x1F),
    (AIEN_OFF|DIFF_DIFFERENTIAL|0x1F));
        
    for(;;){
      ADC3SC1A_ADCHA = 0x00;
    
    	do{
    	/* Wait for conversion to complete */
    	}while (ADC3SC2_ADACT == 1);	
      
      data = ADC3RA;
    } /* loop forever */
  }
  
  if (DADn == 3){
    /* Configure ADC4 */
    ADCPointer = (tADC *)ADC4;
    ADC_Init(ADCPointer, (ADLPC_NORMAL|ADIV_8|ADLSMP_SHORT|MODE|ADICLK_BUS_2), (ADACKEN_ENABLED|ADHSC_HISPEED|ADLSTS_2) , 0, 0,
    (ADTRG_SW|ACFE_DISABLED|ACFGT_LESS|ACREN_DISABLED|REFSEL_EXT), (ADCO_SINGLE|AVGE_DISABLED), (AIEN_OFF|DIFF_DIFFERENTIAL|0x1F),
    (AIEN_OFF|DIFF_DIFFERENTIAL|0x1F));
        
    for(;;){
      ADC4SC1A_ADCHA = 0x03;
          	
    	do{
    	/* Wait for conversion to complete */
    	}while (ADC4SC2_ADACT == 1);	
      
      data = ADC4RA;
    } /* loop forever */
  } 
} 

/******************************************************************************
Function Name	:	System_Init
Engineer      :	r54940
Date          :	12/09/08

Parameters		:	
Returns			  :	NONE
Notes         :	Configures system registers and I/O
******************************************************************************/
void System_Init(void)
{

  IRQSC = OFF;
  SPMSC1 = OFF;
  SPMSC2 = OFF;
  SPMSC3 = OFF;
  SOPT1 = 0x20;          /* Enable Stop Mode, Disable COP */
  INTC_WCR = 0x86;       /* Enable all levels of interrupts */
  SCGC1 = 0xF7;          /* ADC's and SCI's gated on */
  SCGC2 = 0x20;          /* Vref Gated on */
  SCGC3 = OFF;
  SCGC4 = 0x08;          /* PDB gated on */
  SCGC5 = 0xFC;
  SIMIPS1 = OFF;
  SIMIPS2 = OFF;
  
  /* disable all pull ups */
  PTAPE = ON;
  PTBPE = ON;
  PTCPE = ON;
  PTDPE = ON;
  PTEPE = ON;
  PTFPE = ON;
  
  /* low drive strength */
  PTADS = OFF;
  PTBDS = OFF;
  PTCDS = OFF;
  PTDDS = OFF;
  PTEDS = OFF;
  PTFDS = OFF;
  
  /* disable slew rate */
  PTASE = OFF;
  PTBSE = OFF;
  PTCSE = OFF;
  PTDSE = OFF;
  PTESE = OFF;
  PTFSE = OFF;
  
  /* enable input filters */
  PTAIFE = ON;
  PTBIFE = ON;
  PTCIFE = ON;
  PTDIFE = ON;
  PTEIFE = ON;
  PTFIFE = ON;
  
  PTAD = CLEAR;
  PTBD = CLEAR;
  PTCD = CLEAR;
  PTDD = CLEAR;
  PTED = CLEAR;
  PTFD = CLEAR;
  
  PTADD = SET;
  PTBDD = SET;
  PTCDD = SET;
  PTDDD = SET;
  PTEDD = SET;
  PTFDD = SET;
  
  /* enable all ADC pins via mux control register */
  PTAPF1 = 0xFC;
  PTAPF2 = 0x30;
  PTBPF1 = 0xF0;
  PTBPF2 = 0x00;
  PTCPF1 = 0x00;
  PTCPF2 = 0x00;
  PTDPF1 = 0x00;
  PTDPF2 = 0x00;
  PTEPF1 = 0x00;
  PTEPF2 = 0x0F;
  PTFPF1 = 0xFF;
  PTFPF2 = 0x00;

}
